package com.axel.speedm;

import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.app.Activity;
import android.content.Context;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.TextView;

public class MainActivity extends Activity implements SensorEventListener {

	@SuppressWarnings("unused")
	private SensorManager sensorManager; 	
	private LocationManager locationManager;
	private LocationListener locationListener = new MyLocationListener();
	
	private TextView status;
	private TextView mainOut;
	private TextView maxOut;
	private TextView minOut;
	private Button reset; 
	private Button kmh;
	private Button ms; 
	
	private final double MIN_SPEED = 999;			// maximum and minimum values are in meters/second
	private final double MAX_SPEED = 0;
		
	private double speed = 0; 
	private double maxSpeed = MAX_SPEED; 
	private double minSpeed = MIN_SPEED;

	
	private final int DISPLAY_MODE_KMH = 0;		
	private final int DISPLAY_MODE_MS = 1;
	
	private int displayMode = DISPLAY_MODE_KMH; 
	
    @Override
    public void onCreate(Bundle savedInstanceState) {						// called when the application is started
        super.onCreate(savedInstanceState);
        
        requestWindowFeature(Window.FEATURE_NO_TITLE);						// hide application title
        setContentView(R.layout.activity_main);
        
        kmh = (Button) findViewById(R.id.mode_kmh); 						// import textViews and buttons
        ms = (Button) findViewById(R.id.mode_ms); 
        mainOut = (TextView) findViewById(R.id.main_output);
        maxOut = (TextView) findViewById(R.id.max_speed_out);
        minOut = (TextView) findViewById(R.id.min_speed_out);        
        reset = (Button) findViewById(R.id.reset_button);        
        status = (TextView) findViewById(R.id.status_text);
        
        
        mainOut.setText("" + 0.0);										// set tests to default values
		maxOut.setText("" + 0.0);
		minOut.setText("" + 0.0);
		
		status.setTextColor(Color.RED);
        status.setText("Waiting for GPS signal...");
        
        
        reset.setOnClickListener(new View.OnClickListener() {				// reset button listener
			public void onClick(View v) {			
				status.setTextColor(Color.RED);
				status.setText("Reseting maximum and minimum values...");
				maxSpeed = MAX_SPEED; 
				minSpeed = MIN_SPEED;
			}
		}); 
        
        kmh.setOnClickListener(new View.OnClickListener() {					// kilometers/hour mode button listener
			public void onClick(View v) {		
				status.setTextColor(Color.YELLOW);
				status.setText("Changing mode to km/h...");
				displayMode = DISPLAY_MODE_KMH; 
				maxSpeed = MAX_SPEED; 
				minSpeed = MIN_SPEED;
			}
		}); 
        
        ms.setOnClickListener(new View.OnClickListener() {					// meters/second mode button listener
			public void onClick(View v) {		
				status.setTextColor(Color.YELLOW);
				status.setText("Changing mode to m/s...");
				displayMode = DISPLAY_MODE_MS; 
				maxSpeed = MAX_SPEED; 
				minSpeed = MIN_SPEED;
			}
		}); 

        locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, locationListener);     

        sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
    }
    
	@Override
	protected void onResume() {												// when the devise is unlocked and the program goes active again, 
		super.onResume(); 													// register this class as a listener for the orientation and accelerometer sensors
	}	
	  
	@Override
	protected void onPause() {												// called when the device goes to sleep, unregister the orientation listener 	    				
		super.onPause();													// note that the GPS is still on!
	}
	
	public void onAccuracyChanged(Sensor sensor, int accuracy) {			// called when update frequency ore something else is updated
		// TODO Auto-generated method stub		
	}

	public void onSensorChanged(SensorEvent event) {					
		// TODO Auto-generated method stub		
	}
	
	class MyLocationListener implements LocationListener 					// this class uses a location listener to access data from the GPS
	{
		public void onLocationChanged(Location location) 					// is called when a position update is received, once every second
		{
			if (location != null) 											// if positions-data is available
			{		  	
				speed = (double) location.getSpeed();
							
				status.setTextColor(Color.GREEN);							// display signal OK and GPS accuracy
				status.setText("GPS signal OK!    GPS Accuracy: " + (int) location.getAccuracy() + " m");
				
				if (displayMode == DISPLAY_MODE_KMH) {						// convert to kilometers/hour
					speed *= 3.6; 
				}
								
				speed = Math.round(speed * 10) / 10.0d; 					// round to one decimal
				
				if (speed > maxSpeed) {										// save maximum speed
					maxSpeed = speed;
					maxOut.setTextColor(Color.YELLOW);
				} else {
					maxOut.setTextColor(Color.WHITE);
				}
				
				if (speed < minSpeed) {										// save minimum speed
					minSpeed = speed;
					minOut.setTextColor(Color.YELLOW);
				} else {
					minOut.setTextColor(Color.WHITE);
				}					
				
				if (speed > 999.9) {										// limit speed to max 999.9
					speed = 999.9;
				}
				
				if (speed < 0) {											// no negative speeds allowed
					speed = 0;
				}
				
				mainOut.setText("" + speed);								// display the speeds
				maxOut.setText("" + maxSpeed);
				minOut.setText("" + minSpeed);
			} 
			else {
				status.setTextColor(Color.RED);								// if the position is invalid
				status.setText("Waiting for GPS signal...");
			}
		}
		  
		public void onProviderDisabled(String provider) {					// not used
			// TODO Auto-generated method stub		
		}	
		
		public void onProviderEnabled(String provider){						// not used
			// TODO Auto-generated method stub		
		}
			
		public void onStatusChanged(String provider, int status, Bundle extras) {		// not used
			// TODO Auto-generated method stub		
		}
	}
}
